@@ -123,6 +123,21 @@ tf.flags.DEFINE_string(
     'window_title', '',
     'The title of the browser window.')
 
+tf.flags.DEFINE_integer(
+    'max_reload_threads', 1,
+    'The max number of threads that TensorBoard can use to reload runs. Not '
+    'relevant for db mode. Each thread reloads one run at a time.')
+
+tf.flags.DEFINE_string(
+    'samples_per_plugin', '', 'An optional comma separated list of '
+    'plugin_name=num_samples pairs to explicitly specify how many samples to '
+    'keep per tag for that plugin. For unspecified plugins, TensorBoard '
+    'randomly downsamples logged summaries to reasonable values to prevent '
+    'out-of-memory errors for long running jobs. This flag allows fine control '
+    'over that downsampling. Note that 0 means keep all samples of that type. '
+    'For instance, "scalars=500,images=0" keeps 500 scalars and all images. '
+    'Most users should not need to set this flag.')
+
 FLAGS = tf.flags.FLAGS
 
 
@@ -187,8 +202,10 @@ def create_tb_app(plugins, assets_zip_provider=None):
   :rtype: types.FunctionType
   """
   if not FLAGS.db and not FLAGS.logdir:
-    raise ValueError('A logdir must be specified when db is not specified. '
-                     'Run `tensorboard --help` for details and examples.')
+    raise ValueError('A logdir or db must be specified. '
+                     'For example `tensorboard --logdir mylogdir` '
+                     'or `tensorboard --db sqlite:~/.tensorboard.db`. '
+                     'Run `tensorboard --helpfull` for details and examples.')
   return application.standard_tensorboard_wsgi(
       assets_zip_provider=assets_zip_provider,
       db_uri=FLAGS.db,
@@ -198,6 +215,7 @@ def create_tb_app(plugins, assets_zip_provider=None):
       plugins=plugins,
       path_prefix=FLAGS.path_prefix,
       window_title=FLAGS.window_title,
+      max_reload_threads=FLAGS.max_reload_threads,
       flags=FLAGS)
 
 
